/*
 * Cylon API
 * cylonjs.commap
 * Copyright (c) 2013-2015 The Hybrid Group
 * Licensed under the Apache 2.0 license.
*/

'use strict';

var socketIO = require('socket.io'),
    _ = require('lodash');

var SocketMaster = module.exports = function SocketMaster(http, mcp) {
  this.http = http;
  this.mcp = mcp;
  this.io = {};
  this.nsp = {};
};

SocketMaster.prototype.start = function() {
  this.io = this._io();

  console.log('Socket.io has started...');

  this.io.on('connection', function(socket) {
    console.log('A user connected');

    socket.on('disconnect', function() {
      console.log('A user disconnected');
    });

    this.socketMCP();
    this.socketRobots(this.mcp.robots);

    _.forIn(this.mcp.robots, function(robot) {
      this.socketDevices(robot);
    }, this);
  }.bind(this));
};

SocketMaster.prototype._socketItems = function(nsp, items, callback) {
  _.forIn(items, function(item, name) {
    var nspName;

    name = name.toLowerCase();
    nspName = nsp + name;

    if (!this.nsp[name]) {
      console.log('Creating new socket for: ' + nspName);

      this.nsp[name] = this.io.of(nspName);

      this.nsp[name].on('connection', function(socket) {
        console.log('Connected to socket: ', nspName);

        socket.on('disconnect', function() {
          console.log('disconnected from socket: ', nspName);
        });

        callback(socket, name, item);
      }.bind(this));
    }
  }, this);
};

SocketMaster.prototype.socketMCP = function() {
  console.log('Setting up API socket...');

  var callback = function(socket, name, robots) {
    var robotNames = _.map(robots, function(val, key) {
      return key.toLowerCase();
    });

    socket.on('robots', function() {
      this.nsp[name].emit('robots', robotNames);
    }.bind(this));

    this.nsp[name].emit('robots', robotNames);
  }.bind(this);

  this._socketItems('/api/', { robots: this.mcp.robots }, callback);
};

SocketMaster.prototype.socketRobots = function(robots) {
  console.log('Setting up robot sockets...');

  var callback = function(socket, name, robot) {
    var devices = _.keys(robot.devices);

    socket.on('devices', function() {
      this.nsp[name].emit('devices', devices);
    }.bind(this));

    this._addDefaultListeners(socket, name, robot);

    this.nsp[name].emit('devices', devices);
  }.bind(this);

  this._socketItems('/api/robots/', robots, callback);
};

SocketMaster.prototype.socketDevices = function(robot) {
  console.log('Setting up device sockets...');

  var robotName = robot.name.toLowerCase(),
      nsp = '/api/robots/' + robotName + '/devices/';

  this._socketItems(nsp, robot.devices, this._addDefaultListeners.bind(this));
};

SocketMaster.prototype._addDefaultListeners = function(socket, name, item) {
  socket.on('message', function(msg) {
    this.nsp[name].emit('message', msg);
  }.bind(this));

  socket.on('commands', function() {
    var commands = _.keys(item.commands);
    this.nsp[name].emit('commands', commands);
  }.bind(this));

  socket.on('events', function() {
    var events = item.events;
    this.nsp[name].emit('events', events);
  }.bind(this));

  socket.on('command', function(opts) {
    var ret = item.commands[opts.command].apply(item, opts.args);
    this.nsp[name].emit('command', opts.command, ret);
  }.bind(this));

  _.forIn(item.commands, function(command, cname) {
    socket.on(cname, function() {
      var ret = command.apply(item, arguments);
      this.nsp[name].emit(cname, ret);
    }.bind(this));
  }, this);

  _.forIn(item.events, function(eventName) {
    item.on(eventName, function() {
      var args = _.toArray(arguments),
          nsp = this.nsp[name];

      args.unshift(eventName);

      nsp.emit.apply(nsp, args);
    }.bind(this));
  }, this);
};

SocketMaster.prototype._io = function() {
  return socketIO(this.http);
};
